import { world, EquipmentSlot, EntityEquippableComponent, system } from "@minecraft/server";


const TOOL_HAMMER = "capita:hammer";    // RockCapitator
const TOOL_DRILL = "capita:drill";     // DigCapitator
const TOOL_CLEANER = "capita:cleaner";   // SnowCapitator
const TOOL_HARVESTER = "capita:harvester"; // FlowerCapitator


const ROCK_TYPES = [
   "minecraft:stone", "minecraft:granite", "minecraft:polished_granite",
   "minecraft:andesite", "minecraft:polished_andesite", "minecraft:diorite",
   "minecraft:polished_diorite", "minecraft:cobbled_deepslate", "minecraft:deepslate",
   "minecraft:polished_deepslate", "minecraft:tuff"
];
const DIG_TYPES = [
   "minecraft:dirt", "minecraft:coarse_dirt", "minecraft:grass_block",
   "minecraft:gravel", "minecraft:sand", "minecraft:red_sand", "minecraft:clay"
];
const SNOW_TYPES = [
   "minecraft:snow_layer",
   "minecraft:snow",
   "minecraft:ice",
   "minecraft:packed_ice",
   "minecraft:blue_ice",
   "minecraft:powder_snow"
];
const FLOWER_TYPES = [
   "minecraft:dandelion", "minecraft:poppy", "minecraft:azure_bluet",
   "minecraft:oxeye_daisy", "minecraft:allium", "minecraft:blue_orchid",
   "minecraft:cornflower", "minecraft:lily_of_the_valley",
   "minecraft:wither_rose", "minecraft:sunflower", "minecraft:lilac",
   "minecraft:rose_bush", "minecraft:peony",
   "minecraft:brown_mushroom", "minecraft:red_mushroom",
   "minecraft:fungus", "minecraft:warped_roots", "minecraft:crimson_roots",
   "minecraft:fern", "minecraft:large_fern",
   "minecraft:grass", "minecraft:tall_grass", "minecraft:tall_dry_grass", "minecraft:tall_grassshort_dry_grass", "minecraft:short_grass", "minecraft:seagrass"
];


const MAX_H_DIST = 6;
const MAX_V_DIST = 6;
const HARVEST_RADIUS = 12;

function findConnected(origin, dimension, validTypes) {
   const queue = [origin];
   const seen = new Set();
   const found = [];

   while (queue.length) {
      const loc = queue.pop();
      const key = `${loc.x},${loc.y},${loc.z}`;
      if (seen.has(key)) continue;
      seen.add(key);

      const blk = dimension.getBlock(loc);
      if (!blk || !validTypes.includes(blk.typeId)) continue;
      found.push(loc);


      for (let dx = -1; dx <= 1; dx++) {
         for (let dy = -1; dy <= 1; dy++) {
            for (let dz = -1; dz <= 1; dz++) {
               if (dx === 0 && dy === 0 && dz === 0) continue;
               const nb = { x: loc.x + dx, y: loc.y + dy, z: loc.z + dz };
               if (
                  Math.abs(nb.x - origin.x) <= MAX_H_DIST &&
                  Math.abs(nb.y - origin.y) <= MAX_V_DIST &&
                  Math.abs(nb.z - origin.z) <= MAX_H_DIST
               ) queue.push(nb);
            }
         }
      }
   }

   return found;
}


function forEachInCube(origin, radius, fn) {
   for (let dx = -radius; dx <= radius; dx++) {
      for (let dy = -radius; dy <= radius; dy++) {
         for (let dz = -radius; dz <= radius; dz++) {
            fn({ x: origin.x + dx, y: origin.y + dy, z: origin.z + dz });
         }
      }
   }
}


world.beforeEvents.playerBreakBlock.subscribe(({ block, player, cancel }) => {
   try {
      const equip = player.getComponent(EntityEquippableComponent.componentId);
      const tool = equip?.getEquipment(EquipmentSlot.Mainhand);
      if (!tool) return;

      const tid = tool.typeId;
      const bid = block.typeId;
      let mode = null;
      let validList = null;
      let offTag = "";

      if (tid === TOOL_HAMMER && ROCK_TYPES.includes(bid)) { mode = "flood"; validList = ROCK_TYPES; offTag = "hammerOff"; }
      if (tid === TOOL_DRILL && DIG_TYPES.includes(bid)) { mode = "flood"; validList = DIG_TYPES; offTag = "drillOff"; }
      if (tid === TOOL_CLEANER && SNOW_TYPES.includes(bid)) { mode = "flood"; validList = SNOW_TYPES; offTag = "cleanerOff"; }
      if (tid === TOOL_HARVESTER && FLOWER_TYPES.includes(bid)) { mode = "radius"; validList = FLOWER_TYPES; offTag = "harvesterOff"; }

      if (!mode || player.hasTag(offTag)) return;


      cancel = true;
      const dim = block.dimension;
      const origin = block.location;
      let count = 0;

      if (mode === "flood") {
         const connected = findConnected(origin, dim, validList);
         for (const loc of connected) {

            dim.runCommandAsync(`setblock ${loc.x} ${loc.y} ${loc.z} air destroy`);
            player.runCommandAsync(`particle new:crit ${loc.x} ${loc.y} ${loc.z}`);
            count++;
         }
      } else {
         forEachInCube(origin, HARVEST_RADIUS, loc => {
            const b = dim.getBlock(loc);
            if (b && validList.includes(b.typeId)) {
               dim.runCommandAsync(`setblock ${loc.x} ${loc.y} ${loc.z} air destroy`);
               player.runCommandAsync(`particle new:crit ${loc.x} ${loc.y} ${loc.z}`);
               count++;
            }
         });
      }

      player.runCommandAsync(`playsound random.pop @s`);
      player.runCommandAsync(`title @s subtitle §aProcessed: ${count} blocks`);
   } catch (e) {
      console.error("ExtraCapitators error:", e);
   }
});


world.afterEvents.itemUse.subscribe(({ source: player, itemStack: item }) => {
   try {
      if (!player.isSneaking) return;
      const tid = item.typeId;
      let tag = "", name = "";

      if (tid === TOOL_HAMMER) { tag = "hammerOff"; name = "RockCapitato"; }
      if (tid === TOOL_DRILL) { tag = "drillOff"; name = "DigCapitator"; }
      if (tid === TOOL_CLEANER) { tag = "cleanerOff"; name = "SnowCapitator"; }
      if (tid === TOOL_HARVESTER) { tag = "harvesterOff"; name = "ShearCapitator"; }

      if (!tag) return;
      if (player.hasTag(tag)) {
         player.removeTag(tag);
         player.runCommandAsync(`title @s actionbar §a[${name}] ON`);
      } else {
         player.addTag(tag);
         player.runCommandAsync(`title @s actionbar §c[${name}] OFF`);
      }
   } catch { }
});


system.runInterval(() => {
   for (const p of world.getPlayers()) {
      try {
         const eq = p.getComponent(EntityEquippableComponent.componentId);
         const hand = eq?.getEquipment(EquipmentSlot.Mainhand)?.typeId;
         const isTool = [
            TOOL_HAMMER, TOOL_DRILL,
            TOOL_CLEANER, TOOL_HARVESTER
         ].includes(hand);

         if (isTool && !p.hasTag("extraNotif")) {
            p.runCommandAsync(
               `title @s actionbar §6[ExtraCapitators] Break → area. Sneak+Use to toggle`
            );
            p.addTag("extraNotif");
         }
         if (!isTool && p.hasTag("extraNotif")) p.removeTag("extraNotif");
      } catch {
         p.hasTag("extraNotif") && p.removeTag("extraNotif");
      }
   }
}, 20);
